@@ -1,10 +1,9 @@
 /*
-    author: John Hughes (jhughes@frostburg.edu)
-    date: 11/01
+      author: John Hughes (jhughes@frostburg.edu)
+    modified: 3/16/02
 */
 
 #include "platform.h"
-#include <string.h>    /* We need this for strlen. */
 
 /*
     function: rc6_generateKeySchedule
@@ -19,7 +18,7 @@ void rc6_generateKeySchedule(unsigned char* initKey,
                             )
 {
 	unsigned int L[8];  /* We need up to 32 bytes. */
-    unsigned int A, B, i, j, s, v;
+    register unsigned int A, B, i, j, s, v;
 
     /* Point to the lowest byte of L. */
 
@@ -27,8 +26,7 @@ void rc6_generateKeySchedule(unsigned char* initKey,
     
     /* Move the bytes of initKey into L, little-endian fashion. */
 
-	for (j = 0; j < keyLength; j++)
-		*bPtr++ = initKey[j];
+    memcpy(bPtr, initKey, keyLength);
 
     /* Set S[0] to the constant P32, then generate the rest of S. */
 
@@ -36,8 +34,8 @@ void rc6_generateKeySchedule(unsigned char* initKey,
     for (i = 1; i < 44; i++)
 		S[i] = S[i - 1] + 0x9E3779B9;
     A = B = i = j = 0;
-	v = 3 * 44;
-    keyLength /= 4; 
+	v = 132;
+    keyLength >>= 2; 
 	for (s = 1; s <= v; s++)
 	{
 		A = S[i] = ROL(S[i] + A + B, 3);
@@ -51,42 +49,43 @@ void rc6_generateKeySchedule(unsigned char* initKey,
     function: rc6_encrypt
 
     description: This function takes a 16-byte block and encrypts it into 
-                 'output.'
+                 'output'.
 */
 
 void rc6_encrypt(unsigned char* input, unsigned int S[], unsigned char* output)
 {
-    unsigned int* A,* B,* C,* D;
+    register unsigned int A, B, C, D;
     unsigned int regs[4];
-    unsigned int t, u, temp, j;
+    register unsigned int t, u, temp, j;
 	unsigned char* regPtr;
 
-    A = &regs[0]; /* Cook up *A, *B, *C, and *D as our four 32-bit registers. */
-    B = &regs[1];
-    C = &regs[2];
-    D = &regs[3];
-    regPtr = (unsigned char*)A;
-	for (j = 0; j < 16; j++)     /* Move the input into the registers. */
-        *regPtr++ = *input++;
-    regPtr -= 16;
-	*B += S[0];
-	*D += S[1];
+    regPtr = (unsigned char*)&regs[0];
+    memcpy(regPtr, input, 16);
+    A = regs[0]; /* Cook up A, B, C, and D as our four 32-bit registers. */
+    B = regs[1];
+    C = regs[2];
+    D = regs[3];
+	B += S[0];
+	D += S[1];
 	for (j = 1; j <= 20; j++)  /* Perform 20 rounds. */
 	{
-	    t = ROL(*B * (2 * *B + 1), 5);
-        u = ROL(*D * (2 * *D + 1), 5);
-        *A = ROL(*A ^ t, u) + S[ 2 * j];
-        *C = ROL(*C ^ u, t) + S[2 * j + 1];
-		temp = *A;
-		*A = *B;
-		*B = *C;
-		*C = *D;
-		*D = temp;
+	    t = ROL(B * ((B << 1) + 1), 5);
+        u = ROL(D * ((D << 1) + 1), 5);
+        A = ROL(A ^ t, u) + S[j << 1];
+        C = ROL(C ^ u, t) + S[(j << 1) + 1];
+		temp = A;
+		A = B;
+		B = C;
+		C = D;
+		D = temp;
     }
-	*A += S[42];
-	*C += S[43];
-	for (j = 0; j < 16; j++)
-	    *output++ = *regPtr++;
+	A += S[42];
+	C += S[43];
+    regs[0] = A;
+    regs[1] = B;
+    regs[2] = C;
+    regs[3] = D;
+    memcpy(output, regPtr, 16);
 }
 
 /*
@@ -98,38 +97,39 @@ void rc6_encrypt(unsigned char* input, unsigned int S[], unsigned char* output)
 
 void rc6_decrypt(unsigned char* input, unsigned int S[], unsigned char* output)
 {
-    unsigned int* A,* B,* C,* D;
+    register unsigned int A, B, C, D;
     unsigned int regs[4];
-	unsigned int t, u, temp, temp2, j;
+	register unsigned int t, u, temp, temp2, j;
 	unsigned char* regPtr;
 
-	A = &regs[0];
-    B = &regs[1];
-    C = &regs[2];
-    D = &regs[3];
-    regPtr = (unsigned char*)A;
-	for (j = 0; j < 16; j++)
-	    *regPtr++ = *input++;
-	*C -= S[43];
-	*A -= S[42];
+    regPtr = (unsigned char*)&regs[0];
+    memcpy(regPtr, input, 16);
+    A = regs[0];
+    B = regs[1];
+    C = regs[2];
+    D = regs[3];
+	C -= S[43];
+	A -= S[42];
 	for (j = 20; j >= 1; j--)
 	{
-	    temp = *A;
-		*A = *D;
-		temp2 = *B;
-		*B = temp;
-		temp = *C;
-		*C = temp2;
-		*D = temp;
-		t = ROL(*B * (2 * *B + 1), 5);
-		u = ROL(*D * (2 * *D + 1), 5);
-		*A = ROR(*A - S[2 * j], u) ^ t;
-		*C = ROR(*C - S[2 * j + 1], t) ^ u;
+	    temp = A;
+		A = D;
+		temp2 = B;
+		B = temp;
+		temp = C;
+		C = temp2;
+		D = temp;
+		t = ROL(B * ((B << 1) + 1), 5);
+		u = ROL(D * ((D << 1) + 1), 5);
+		A = ROR(A - S[j << 1], u) ^ t;
+		C = ROR(C - S[(j << 1) + 1], t) ^ u;
 	}
-	*D -= S[1];
-	*B -= S[0];
-	regPtr = (unsigned char*)A;
-	for (j = 0; j < 16; j++)
-	    *output++ = *regPtr++;
+	D -= S[1];
+	B -= S[0];
+    regs[0] = A;
+    regs[1] = B;
+    regs[2] = C;
+    regs[3] = D;
+    memcpy(output, regPtr, 16);
 }
 
